<?php

/**
 * Plugin Name: Attachments UI
 * Plugin URI:  http://mondaybynoon.com/members/plugins/attachments-ui/
 * Description: An extension for Attachments that provides a UI for implementing Instances and an easy way of generating location-based rules for them
 * Author:      Jonathan Christopher
 * Author URI:  http://mondaybynoon.com/
 * Version:     1.0.1
 * Text Domain: attachmentsui
 * Domain Path: /languages/
 * License:     GPLv2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */

// exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

define( 'ATTACHMENTS_UI_DIR', plugin_dir_path( __FILE__ ) );
define( 'ATTACHMENTS_UI_URL', plugins_url( 'attachments-ui' ) );



/**
 * Attachments UI
 * An extension for Attachments that provides a UI for implementing Instances and
 * an easy way of generating location-based rules for them
 *
 * @package Attachments
 * @subpackage AttachmentsUI
 */
class AttachmentsUI
{

    private $version;
    private $url;
    private $dir;
    private $meta_key       = 'attachmentsui';
    private $instance_key   = 'aui_instance_';
    private $cpt_name       = 'attachmentsui';
    private $parent;

    private $magic_tag_attachments;
    private $magic_tag_index;



    /**
     * Constructor
     *
     * @since 1.0
     */
    function __construct( $attachments = null )
    {
        if( !is_null( $attachments ) )
        {
            $this->version  = '1.0.1';
            $this->url      = ATTACHMENTS_UI_URL;
            $this->dir      = ATTACHMENTS_UI_DIR;
            $this->parent   = $attachments;
            $plugin         = plugin_basename( __FILE__ );

            // WordPress hooks
            add_action( 'init',                         array( $this, 'wp_init' ) );
            add_action( 'admin_menu',                   array( $this, 'bootloader' ) );
            add_filter( "plugin_action_links_$plugin",  array( $this, 'plugin_settings_link' ) );

            // Attachments hooks
            add_action( 'attachments_register', array( $this, 'register_instances' ) );

            if( !is_admin() )
                add_filter( 'the_content', array( $this, 'auto_append' ) );
        }
    }



    function plugin_settings_link( $links )
    {
        $settings_link = '<a href="edit.php?post_type=attachmentsui">'. __( 'Manage', 'attachmentsui' ) . '</a>';
        array_unshift( $links, $settings_link );
        return $links;
    }




    /**
     * Callback for init action
     */
    function wp_init()
    {
        $this->register_cpt();
    }



    /**
     * Bootloader for Attachments UI, executes once Attachments has been instantiated
     */
    function bootloader()
    {
        if( is_plugin_active( 'attachments/index.php' ) && class_exists( 'Attachments' ) )
        {
            $this->wp_admin_menu();  // we've already piggybacked admin_menu to get here
                                     // so as to have is_plugin_active() available to us

            add_action( 'admin_enqueue_scripts', array( $this, 'assets' ), 999, 1 );

            add_action( 'add_meta_boxes', array( $this, 'wp_add_meta_boxes' ) );
            add_action( 'save_post', array( $this, 'save' ) );
        }
    }



    /**
     * Pulls CPT entries we're using to store instance settings and tells Attachments to use them
     */
    function register_instances()
    {
        $args = array(
            'posts_per_page'    => -1,
            'numberposts'       => -1,
            'post_type'         => $this->cpt_name
        );

        $instances = get_posts( $args );

        foreach( $instances as $instance )
        {
            $metadata   = get_post_meta( $instance->ID, $this->instance_key . $instance->ID, true );
            $settings   = isset( $metadata['settings'] ) ? $metadata['settings'] : array();
            $fields     = isset( $metadata['fields'] ) ? $metadata['fields'] : array();
            $locations  = isset( $metadata['location'] ) ? $metadata['location'] : array();

            // build our post types
            $post_types = array();
            if( isset( $settings['post_type'] ) )
                foreach( $settings['post_type'] as $post_type )
                    $post_types[] = $post_type;

            // build our file types
            $file_types = array();
            if( isset( $settings['filetype'] ) )
                foreach( $settings['filetype'] as $file_type )
                    $file_types[] = $file_type;

            // we need to parse our $fields to check for meta
            if( count( $fields ) )
            {
                foreach( $fields as $field_key => $field )
                {
                    if( isset( $field['meta'] ) && is_array( $field['meta'] ) && count( $field['meta'] ) )
                    {
                        foreach( $field['meta'] as $field_meta_key => $field_meta_value )
                        {
                            // bools are stored as strings...
                            if( is_numeric( $field_meta_value ) )
                                $field_meta_value = (bool) $field_meta_value;

                            switch( gettype( $field_meta_value ) )
                            {
                                case 'array':
                                case 'string' :
                                    $field_options = array();
                                    $options = explode( "\n", $field_meta_value );
                                    if( is_array( $options ) )
                                    {
                                        foreach( $options as $option )
                                        {
                                            $pair = explode( ':', $option );
                                            if( is_array( $pair ) && count( $pair ) == 2 )
                                            {
                                                $field_options[trim( $pair[0] )] = trim( $pair[1] );
                                            }
                                        }
                                        $fields[$field_key]['meta'][$field_meta_key] = $field_options;
                                    }
                                    break;

                                case 'boolean':
                                    $fields[$field_key]['meta'][$field_meta_key] = (bool) $field_meta_value;
                                    break;
                            }
                        }
                    }
                }
            }

            $instance_name = empty( $metadata['name'] ) ? $this->instance_key . $instance->ID : $metadata['name'];
            $instance_args = array(
                'label'         => get_the_title( $instance->ID ),
                'post_type'     => $post_types,
                'position'      => isset( $settings['position'] ) ? $settings['position'] : '',
                'priority'      => isset( $settings['priority'] ) ? $settings['priority'] : '',
                'limit'         => isset( $settings['limit'] ) ? intval( $settings['limit'] ) : -1,
                'filetype'      => $file_types,
                'note'          => isset( $settings['note'] ) ? $settings['note'] : '',
                'append'        => isset( $settings['append'] ) ? true : false,
                'button_text'   => isset( $settings['button_text'] ) ? $settings['button_text'] : '',
                'modal_text'    => isset( $settings['modal_text'] ) ? $settings['modal_text'] : '',
                'router'        => isset( $settings['router'] ) ? $settings['router'] : '',
                'fields'        => $fields,
            );

            // fire the register() method for this Instance
            $this->parent->register( $instance_name, $instance_args );

            // we'll also implement any Location Limits that might be in place
            if( !empty( $locations ) )
				add_filter( "attachments_location_{$instance_name}", array( $this, 'check_rules' ), 10, 2 );

        }
    }



    /**
     * Callback to implement our JavaScript and CSS assets when appropriate
     *
     * @param string $hook Provided by WordPress
     */
    function assets( $hook )
    {
        global $post;

        // we only want our assets on edit screens
        if( 'edit.php' != $hook && 'post.php' != $hook && 'post-new.php' != $hook )
            return;

        if( isset( $post->post_type ) && $post->post_type != $this->cpt_name )
            return;

        // styles
        wp_enqueue_style( 'attachmentsui',
            trailingslashit( $this->url ) . 'assets/css/style.css',
            null,
            $this->version,
            'screen'
        );

        wp_enqueue_style( 'select2',
            trailingslashit( $this->url ) . 'assets/vendor/select2/select2.css',
            null,
            '3.3.2',
            'screen'
        );

        // scripts
        wp_enqueue_script( 'backbone' );

        wp_enqueue_script( 'select2',
            trailingslashit( $this->url ) . 'assets/vendor/select2/select2.min.js',
            array( 'jquery' ),
            '3.3.2',
            false
        );

        wp_enqueue_script( 'attachmentsui',
            trailingslashit( $this->url ) . 'assets/javascripts/script.js',
            array( 'jquery', 'select2', 'backbone' ),
            $this->version,
            true
        );
    }



    /**
     * Callback for WordPress action add_meta_boxes. Adds our meta boxes for Instance edit screens:
     *      1) Instance Settings
     *      2) Instance Fields
     *      3) Instance Location Limits
     *      4) Code sample for usage
     */
    function wp_add_meta_boxes()
    {
        // Location Limits
        add_meta_box(
            'mb-attachments-ui-location',
            __( 'Location', 'attachmentsui' ),
            array( $this, 'meta_box_markup_location' ),
            $this->cpt_name,
            'normal',
            'high',
            array(

            )
        );

        // Settings
        add_meta_box(
            'mb-attachments-ui-settings',
            __( 'Settings', 'attachmentsui' ),
            array( $this, 'meta_box_markup_settings' ),
            $this->cpt_name,
            'normal',
            'high'
        );

        // Fields
        add_meta_box(
            'mb-attachments-ui-fields',
            __( 'Fields', 'attachmentsui' ),
            array( $this, 'meta_box_markup_fields' ),
            $this->cpt_name,
            'normal',
            'high',
            array(

            )
        );

        // Code Reference
        add_meta_box(
            'mb-attachments-ref',
            __( 'Code Reference', 'attachmentsui' ),
            array( $this, 'meta_box_markup_code_ref' ),
            $this->cpt_name,
            'side',
            'default',
            array(

            )
        );

        // Auto-append
        add_meta_box(
            'mb-attachments-append',
            __( 'Auto-append', 'attachmentsui' ),
            array( $this, 'meta_box_markup_append' ),
            $this->cpt_name,
            'normal',
            'low',
            array(

            )
        );

		// Force instance name
		add_meta_box(
			'mb-attachments-instance-name',
			__( 'Instance Name', 'attachmentsui' ),
			array( $this, 'meta_box_markup_instance_name' ),
			$this->cpt_name,
			'side',
			'low',
			array(

			)
		);
    }



	function meta_box_markup_instance_name( $post )
	{
		$metadata = get_post_meta( $post->ID, $this->instance_key . $post->ID, true );
		$name     = isset( $metadata['name'] ) ? $metadata['name'] : '';

		?>
		<div id="aui-instance-name-override">
			<input type="checkbox" id="<?php echo $this->meta_key; ?>_name_override" name="<?php echo $this->meta_key; ?>[name_override]" value="1"<?php if( !empty( $name ) ) : ?> checked="checked"<?php endif; ?> /> <label for="<?php echo $this->meta_key; ?>_name_override"><?php _e( 'Override default instance name', 'attachmentsui' ); ?></label>
			<div id="aui-instance-name">
				<input type="text" name="<?php echo $this->meta_key; ?>[name]" id="<?php echo $this->meta_key; ?>_name" value="<?php echo $name; ?>" />
			</div>
		</div>
	<?php
	}



    /**
     * Echoes the fields for handling auto-append template
     *
     * @param $post WP_Post WordPress-provided Post object
     */
    function meta_box_markup_append( $post )
    {
        $metadata = get_post_meta( $post->ID, $this->instance_key . $post->ID, true );
        $append = isset( $metadata['append'] ) ? $metadata['append'] : array();

        // Enabled?
        $this->checkbox(
            $this->meta_key . '[append][enabled]',
            __( 'Auto-append', 'attachmentsui' ),
            isset( $append['enabled'] ) ? true : false,
            __( 'Automatically append to <code>the_content()</code> using the template below:', 'attachmentsui' )
        );

        // Template
        $this->textarea(
            $this->meta_key . '[append][template]',
            __( 'Template', 'attachmentsui' ) . '<br /><small><a id="aui-template-toggle" href="#">' . __( 'Reference', 'attachmentsui' ) . '</a></small>',
            isset( $append['template'] ) ? $append['template'] : false
        ); ?>
        <div id="aui-template-reference">
            <?php include 'template-reference.php'; ?>
        </div>
        <?php
    }



    /**
     * Callback for auto-appending template-based output to the_content using Magic Tags
     *
     * @param $content string The existing Post content
     * @return string The populated template content appended to the original $content
     */
    function auto_append( $content )
    {
        $original   = $content;
        $output     = '';

        if( class_exists( 'Attachments' ) )
        {
            // we can use this to grab the instances applicable to the current post
            $instances = $this->parent->get_instances_for_post_type();

            // and then sift through each instance's settings to determine if we need to auto-append or not
            if( is_array( $instances ) && !empty( $instances ) )
            {
                foreach( $instances as $instance )
                {
                    // we need to get the Instance for this post to see if we even want to auto-append
                    $args = array(
                        'posts_per_page'    => 1,
                        'post_type'         => $this->cpt_name,
                        'meta_query'        => array(
                            'key'       => $instance,
                            'compare'   => 'EXISTS',
                        )
                    );

                    $instance_obj = get_posts( $args );

                    if( !empty( $instance_obj ) && isset( $instance_obj[0]->ID ) )
                    {
                        $metadata = get_post_meta( $instance_obj[0]->ID, $instance, true );

                        // if we have metadata we need to process the conditions
                        if( isset( $metadata['append'] ) && isset( $metadata['append']['enabled'] ) && $metadata['append']['enabled'] )
                        {
                            // first we need to get the Attachments themselves
                            $this->magic_tag_attachments = new Attachments( $instance );

                            if( $this->magic_tag_attachments->exist() )
                            {
                                // we want to auto-append using this template:
                                $template = $metadata['append']['template'];

                                // need to grab everything between our begin/end tags
                                $pattern = '/\s*\{@begin_attachments_loop\}\s*(.*?)\s*\{@end_attachments_loop\}\s*/xmis';
                                preg_match( $pattern, $template, $matches );

                                // this loop content will act as the template for all Attachments, we just need to loop through and apply it
                                $loop_content = '';

                                if( isset( $matches[1] ) )
                                {
                                    // we're only going to bother if we in fact have our loop
                                    $loop_content = $matches[1];

                                    $this->magic_tag_index = 1;
                                    while( $this->magic_tag_attachments->get() )
                                    {
                                        $output .= preg_replace_callback( "/({@(.*?)})/m", array( $this, 'parse_magic_tags' ), $loop_content );
                                        $this->magic_tag_index++;
                                    }

                                    // last but not least, we'll swap out the original loop tags (and content within) with the concatenated loop content
                                    $output = preg_replace( $pattern, $output, $template );
                                }
                            }
                        }
                    }
                }
            }
        }

        return $original . $output;
    }



    /**
     * Callback to process all Magic Tags
     *
     * @param $in array Match singleton
     * @return string Magic Tag replaced with actual data
     */
    function parse_magic_tags( $in )
    {
        $name = $in[2];

        switch( $name )
        {
            case 'index':
                $value = $this->magic_tag_index;
                break;

            case 'id':
                $value = $this->magic_tag_attachments->id();
                break;

            case 'thumb':
                $value = $this->magic_tag_attachments->image( 'thumbnail' );
                break;

            case 'url':
                $value = $this->magic_tag_attachments->url();
                break;

            case 'mime':
            case 'type':
                $value = $this->magic_tag_attachments->type();
                break;

            case 'subtype':
                $value = $this->magic_tag_attachments->subtype();
                break;

            case 'filesize':
                $value = $this->magic_tag_attachments->filesize();
                break;

            default:
                $value = '';
                break;
        }

        // we might be dealing with fields
        if( empty( $value ) )
        {
            if( strpos( $name, 'field_' ) !== false )
            {
                // we are dealing with a field
                $field = explode( '_', $name );

                if( isset( $field[1] ) )
                    $value = $this->magic_tag_attachments->field( $field[1] );
            }
        }

        // image details
        if( empty( $value ) )
        {
            $request = explode( '.', $name ); // request format is in the format imagesize.url

            if( is_array( $request ) && count( $request ) == 2 )
            {
                switch ( $request[1] )
                {
                    case 'url':
                    case 'src':
                        $value = $this->magic_tag_attachments->src( $request[0] );
                        break;

                    case 'width':
                        $value = $this->magic_tag_attachments->width( $request[0] );
                        break;

                    case 'height':
                        $value = $this->magic_tag_attachments->height( $request[0] );
                        break;

                    case 'filesize':
                        $value = $this->magic_tag_attachments->filesize( null, $request[0] );
                        break;

                    default:
                        $value = '';
                        break;
                }
            }
        }

        return $value;
    }



    /**
     * Outputs the options for the passed field which represent the attributes specific to that field.
     *      e.g. <options> for <select>s
     *
     * @param null|array $field Existing field or null
     * @param null|string $name The field name or null
     * @param string $array_flag The $_POST array flag to use
     * @param null $fields The registered Attachments fields
     */
    function field_options_markup( $field = null, $name = null, $array_flag = '', $fields = null )
    {
        // if $field is an array, it's an existing field (one that was already saved by the user)
        // but if it's not, it's an Attachments Field object, we can work with attributes of either
        if( is_array( $field ) )
        {
            if( isset( $field['type'] ) && isset( $fields[$field['type']] ) )
            {
                $field_orig = $field;
                $field      = new $fields[$field['type']];

                // we're going to loop through the object meta and populate as necessary
                if( isset( $field_orig['meta'] ) )
                    foreach( $field_orig['meta'] as $field_orig_meta => $value )
                        if( isset( $field->meta[$field_orig_meta] ) )
                            $field->meta[$field_orig_meta] = is_numeric( $value ) ? (bool) $value : $value;

            }
        }

        ?>
        <div class="aui-field-options aui-field-options-<?php echo $name; ?>">
            <?php if( isset( $field->meta ) && is_array( $field->meta ) && !empty( $field->meta ) ) : ?>
                <?php
                foreach( $field->meta as $field_id => $field_meta ) :
                    switch( gettype( $field_meta ) )
                    {
                        case 'array':
                        case 'string': ?>
                            <div class="aui-field-option aui-field-option-select">
                                <label for="aui_fields_<?php echo $array_flag; ?>_<?php echo $field_id; ?>"><?php _e( 'Field Options', 'attachmentsui' ); ?> <span class="description"><?php _e( 'One per line in the format <code>value : label</code>', 'attachmentsui' ); ?></span></label>
                                <textarea name="<?php echo $this->meta_key; ?>[fields][<?php echo $array_flag; ?>][meta][<?php echo $field_id; ?>]" id="aui_fields_<?php echo $array_flag; ?>_<?php echo $field_id; ?>"><?php if( is_string( $field_meta ) ) echo $field_meta; ?></textarea>
                            </div>
                            <?php break;

                        case 'boolean': ?>
                            <div class="aui-field-option aui-field-option-checkbox">
                                <input type="checkbox" name="<?php echo $this->meta_key; ?>[fields][<?php echo $array_flag; ?>][meta][<?php echo $field_id; ?>]" id="aui_fields_<?php echo $array_flag; ?>_<?php echo $field_id; ?>" value="1" <?php checked( intval( $field_meta ), 1 ); ?> />
                                <label for="aui_fields_<?php echo $array_flag; ?>_<?php echo $field_id; ?>"><?php echo $field_id; ?></label>
                            </div>
                            <?php break;

                    }
                endforeach;
                ?>
            <?php else: ?>
                <div class="aui-field-default">
                    <label for="aui_fields_<?php echo $array_flag; ?>_default"><?php _e( 'Field Default', 'attachmentsui' ); ?></label>
                    <select name="<?php echo $this->meta_key; ?>[fields][<?php echo $array_flag; ?>][default]" id="aui_fields_<?php echo $array_flag; ?>_default" style="width:100%;">
                        <option value="0">None</option>
                        <?php foreach( $field->defaults as $default ) : ?>
                            <option value="<?php echo $default; ?>" <?php selected( $field->default, $default ); ?>><?php echo ucfirst( $default ); ?></option>
                        <?php endforeach; ?>
                    </select>
                </div>
            <?php endif; ?>
        </div>
    <?php }



    /**
     * Outputs the main attributes common to all fields e.g. type, label, name
     *
     * @param null|array $field Existing field or null
     * @param null $fields The registered Attachments fields
     * @param string $array_flag The $_POST array flag to use
     */
    function field_markup( $field = null, $fields = null, $array_flag = null )
    { ?>
        <div class="attachments-ui-field-def">
            <div class="attachments-ui-field-handle"><img src="<?php echo trailingslashit( $this->parent->get_url() ) . 'images/handle.gif'; ?>" alt="Handle" width="20" height="20" /></div>
            <div class="attachments-ui-field-delete">
                <a href="#">x</a>
            </div>
            <input type="hidden" name="<?php echo $this->meta_key; ?>[fields][<?php echo $array_flag; ?>][type]" value="<?php if( isset( $field['type'] ) ) : ?><?php echo $field['type']; ?><?php else: ?><%= field %><?php endif; ?>" />
            <div class="aui-field-type aui-field-option">
                <label><?php _e( 'Field Type', 'attachmentsui' ); ?></label>
                <input disabled="disabled" type="text" name="<?php echo $this->meta_key; ?>[fields][<?php echo $array_flag; ?>][type_display]" id="aui_fields_<?php echo $array_flag; ?>_type_display" value="<?php if( isset( $field['type'] ) ) : ?><?php echo $field['type']; ?><?php else: ?><%= field %><?php endif; ?>" />
                <a class="aui-edit-field-settings" href="#"><?php _e( 'Edit Settings', 'attachmentsui' ); ?></a>
            </div>
            <div class="aui-field-label">
                <label for="aui_fields_<?php echo $array_flag; ?>_label"><?php _e( 'Field Label', 'attachmentsui' ); ?></label>
                <input type="text" name="<?php echo $this->meta_key; ?>[fields][<?php echo $array_flag; ?>][label]" id="aui_fields_<?php echo $array_flag; ?>_label" value="<?php if( isset( $field['label'] ) ) echo $field['label']; ?>" />
            </div>
            <div class="aui-field-name">
                <label for="aui_fields_<?php echo $array_flag; ?>_name"><?php _e( 'Field Name', 'attachmentsui' ); ?></label>
                <input type="text" name="<?php echo $this->meta_key; ?>[fields][<?php echo $array_flag; ?>][name]" id="aui_fields_<?php echo $array_flag; ?>_name" value="<?php if( isset( $field['name'] ) ) echo $field['name']; ?>" />
            </div>
            <?php if( is_null( $field ) ) : /* We're generating the template, so we need to cycle through everything */ ?>
                <?php foreach( $fields as $name => $field ) : $my_field = new $field(); ?>
                    <% if( field == '<?php echo $name; ?>' ) { %>
                        <?php $this->field_options_markup( $my_field, $name, $array_flag, $fields ); ?>
                    <% } %>
                <?php endforeach; ?>
            <?php else: ?>
                <?php $this->field_options_markup( $field, null, $array_flag, $fields ); ?>
            <?php endif; ?>
        </div>
    <?php }



    /**
     * Echoes the Underscore template required for all field types and any existing fields for the current Instance.
     * Echoes necessary JavaScript to bind all events for managing fields for the current Instance.
     *
     * @param $post WP_Post Provided by WordPress
     */
    function meta_box_markup_fields( $post )
    {
        $fields                 = $this->parent->get_fields();
        $underscore_template_id = 'tmpl-attachmentsui-field-def';

        $metadata               = get_post_meta( $post->ID, $this->instance_key . $post->ID, true );
        $instance_fields        = isset( $metadata['fields'] ) ? $metadata['fields'] : array();

        if( is_array( $fields ) && count( $fields ) ) : ?>

            <script type="text/template" id="<?php echo $underscore_template_id; ?>">
                <?php $this->field_markup( null, $fields, '<%= field_uid %>' ); ?>
            </script>

            <div id="attachments-ui-fields">

                <div class="attachments-ui-fields-wrapper"><?php if( is_array( $instance_fields ) && count( $instance_fields ) ) : ?>
                    <?php foreach( $instance_fields as $instance_field ) $this->field_markup( $instance_field, $fields, uniqid() ); ?>
                <?php endif; ?></div>

                <div class="attachments-ui-add-field">
                    <ul>
                        <?php foreach( $fields as $name => $field ) : ?>
                            <li>
                                <a data-field="<?php echo esc_attr( $name ); ?>" class="button attachments-ui-add attachments-ui-add-<?php echo $name; ?>"><?php _e( 'Add ' . esc_attr( $name ), 'attachmentsui' ); ?></a>
                            </li>
                        <?php endforeach; ?>
                    </ul>
                </div>

                <script type="text/javascript">
                    jQuery(document).ready(function($){
                        // bind our field additions
                        $('#attachments-ui-fields a.button').click(function(){
                            var $this = $(this);

                            var template = _.template($('script#<?php echo $underscore_template_id; ?>').html());

                            var templateData = {
                                field_uid: attachmentsui_uniqid( 'aui' ),
                                field: $this.data('field')
                            };

                            $('.attachments-ui-fields-wrapper').append(template(templateData));

                            auiSelects();

                            return false;
                        });

                        $('body').on('click', '.attachments-ui-field-delete a', function(){
                            if(confirm('<?php _e( 'Are you sure you want to delete this field?', 'attachmentsui'); ?>')){
                                $(this).parents('.attachments-ui-field-def').remove();
                            }
                            return false;
                        });

                        $('body').on('click', '.aui-edit-field-settings', function(){
                            var fieldOptions = $(this).parents('.attachments-ui-field-def').find('.aui-field-options');
                            if(fieldOptions.is(':visible')){
                                fieldOptions.hide();
                            }else{
                                fieldOptions.show();
                            }
                            return false;
                        });
                    });
                </script>
            </div>
        <?php endif;
    }



    /**
     * Echoes markup for a select
     *
     * @param string $name Field name
     * @param string $label Field label
     * @param array $options Field <option>s
     * @param string $value Field value
     * @param bool $multi Whether the <select> should allow multi
     * @param string $placeholder Field placeholder
     */
    function select( $name, $label, $options, $value, $multi = false, $placeholder = '' )
    { ?>
        <div class="attachments-ui-field attachments-ui-field-select">
            <label for="<?php echo $this->meta_key; ?>_<?php echo $name; ?>"><?php echo $label; ?></label>
            <select name="<?php echo $name; ?><?php if( $multi ) : ?>[]<?php endif; ?>" id="<?php echo $this->meta_key; ?>_<?php echo $name; ?>" <?php if( $multi ) : ?>multiple <?php endif; ?>style="width:65%;"<?php if( !empty( $placeholder ) ) : ?> placeholder="<?php echo esc_attr( $placeholder ); ?>"<?php endif; ?>>
                <?php foreach( $options as $option_value => $label ) : ?>
                    <?php if( is_array( $label ) ) : ?>
                        <optgroup label="<?php echo $label['label']; ?>">
                            <?php if( is_array( $label['options'] ) ) : ?>
                                <?php foreach( $label['options'] as $my_option_value => $my_label ) : ?>
                                    <?php
                                    if( is_array( $value ) )
                                        $selected = in_array( $my_option_value, $value ) ? ' selected="selected"' : '';
                                    else
                                        $selected = selected( $value, $my_option_value, false );
                                    ?>
                                    <option value="<?php echo $my_option_value; ?>" <?php echo $selected; ?>><?php echo $my_label; ?></option>
                                <?php endforeach; ?>
                            <?php endif; ?>
                        </optgroup>
                    <?php else: ?>
                        <?php
                            if( is_array( $value ) )
                                $selected = in_array( $option_value, $value ) ? ' selected="selected"' : '';
                            else
                                $selected = selected( $value, $option_value, false );
                        ?>
                        <option value="<?php echo $option_value; ?>" <?php echo $selected; ?>><?php echo $label; ?></option>
                    <?php endif; ?>
                <?php endforeach; ?>
            </select>
        </div>
    <?php }



    /**
     * Echoes markup for textarea
     *
     * @param string $name Field name
     * @param string $label Field label
     * @param string $value Field value
     */
    function textarea( $name, $label, $value )
    { ?>
        <div class="attachments-ui-field attachments-ui-field-textarea">
            <label for="<?php echo $this->meta_key; ?>_<?php echo $name; ?>"><?php echo $label; ?></label>
            <textarea name="<?php echo $name; ?>" id="<?php echo $this->meta_key; ?>_<?php echo $name; ?>"><?php echo esc_textarea( $value ); ?></textarea>
        </div>
    <?php }



    /**
     * Echoes markup for checkbox
     *
     * @param string $name Field name
     * @param string $label Field label
     * @param string $value Field value
     * @param string $description Field description
     */
    function checkbox( $name, $label, $value, $description )
    { ?>
        <div class="attachments-ui-field attachments-ui-field-checkbox">
            <label for="<?php echo $this->meta_key; ?>_<?php echo $name; ?>"><?php echo $label; ?></label>
            <input type="checkbox" name="<?php echo $name; ?>" id="<?php echo $this->meta_key; ?>_<?php echo $name; ?>" value="1" <?php checked( $value, 1 ); ?> /> <span><?php echo $description; ?></span>
        </div>
    <?php }



    /**
     * Echoes markup for text field
     *
     * @param string $name Field name
     * @param string $label Field label
     * @param string $value Field value
     * @param string $placeholder Field placeholder
     */
    function textfield( $name, $label, $value, $placeholder = '' )
    { ?>
        <div class="attachments-ui-field attachments-ui-field-textfield">
            <label for="<?php echo $this->meta_key; ?>_<?php echo $name; ?>"><?php echo $label; ?></label>
            <input type="text" name="<?php echo $name; ?>" id="<?php echo $this->meta_key; ?>_<?php echo $name; ?>" value="<?php echo esc_attr( $value ); ?>" placeholder="<?php echo $placeholder; ?>" />
        </div>
    <?php }



    /**
     * Echoes the markup for the Settings meta box
     *
     * @param $post WP_Post Provided by WordPress
     */
    function meta_box_markup_settings( $post )
    {
        $metadata = get_post_meta( $post->ID, $this->instance_key . $post->ID, true );
        $settings = isset( $metadata['settings'] ) ? $metadata['settings'] : array();

        wp_nonce_field( 'attachmentsui_save_settings', 'attachmentsui_settings_nonce' );
        ?>
        <div id="attachments-ui-settings">
            <?php
                // Position
                $options = array(
                    'normal'    => 'Normal',
                    'side'      => 'Sidebar',
                    'advanced'  => 'Advanced'
                );

                $this->select(
                    $this->meta_key . '[settings][position]',
                    __( 'Meta Box Position', 'attachmentsui' ),
                    $options,
                    isset( $settings['position'] ) ? $settings['position'] : 'normal'
                );
            ?>
            <?php
                // Priority
                $options = array(
                    'default'   => 'Default',
                    'high'      => 'High',
                    'low'       => 'Low',
                    'core'      => 'Core'
                );

                $this->select(
                    $this->meta_key . '[settings][priority]',
                    __( 'Meta Box Priority', 'attachmentsui' ),
                    $options,
                    isset( $settings['priority'] ) ? $settings['priority'] : 'default'
                );
            ?>
            <?php
                // Limit
                $this->textfield(
                    $this->meta_key . '[settings][limit]',
                    __( 'Limit', 'attachmentsui' ),
                    isset( $settings['limit'] ) ? $settings['limit'] : -1
                );
            ?>
            <?php
                // File Type(s)
                $options = array(
                    'image'         => 'Images',
                    'video'         => 'Videos',
                    'audio'         => 'Audio',
                    'text'          => 'Text',
                    'application'   => 'Application',
                );
                $this->select(
                    $this->meta_key . '[settings][filetype]',
                    __( 'Allowed File Type(s)', 'attachmentsui' ),
                    $options,
                    isset( $settings['filetype'] ) ? $settings['filetype'] : false,
                    true
                );
            ?>
            <?php
                // Note
                $this->textarea(
                    $this->meta_key . '[settings][note]',
                    __( 'Note', 'attachmentsui' ),
                    isset( $settings['note'] ) ? $settings['note'] : false
                );
            ?>
            <?php
                // Append instead of prepend
                $this->checkbox(
                    $this->meta_key . '[settings][append]',
                    __( 'Append', 'attachmentsui' ),
                    isset( $settings['append'] ) ? true : false,
                    __( 'Append new Attachments (disable to prepend)', 'attachmentsui' )
                );
            ?>
            <?php
                // Button Text
                $this->textfield(
                    $this->meta_key . '[settings][button_text]',
                    __( 'Button Text', 'attachmentsui' ),
                    isset( $settings['button_text'] ) ? $settings['button_text'] : __( 'Attach', 'attachments' )
                );
            ?>
            <?php
                // Modal Text
                $this->textfield(
                    $this->meta_key . '[settings][modal_text]',
                    __( 'Modal Text', 'attachmentsui' ),
                    isset( $settings['modal_text'] ) ? $settings['modal_text'] : __( 'Attach', 'attachments' )
                );
            ?>
            <?php
                // Router
                $options = array(
                    'browse'    => 'Browse Existing',
                    'upload'    => 'Upload New'
                );

                $this->select(
                    $this->meta_key . '[settings][router]',
                    __( 'Modal Default', 'attachmentsui' ),
                    $options,
                    isset( $settings['router'] ) ? $settings['router'] : 'browse'
                );
            ?>
        </div>
    <?php }



    /**
     * Echoes the markup for the Location Limits meta box
     *
     * @param $post WP_Post Provided by WordPress
     */
    function meta_box_markup_location( $post )
    {
        global $wp_roles;

        $charset = get_bloginfo( 'charset' );

        $underscore_template_id = 'tmpl-attachmentsui-location-def';

        $metadata   = get_post_meta( $post->ID, $this->instance_key . $post->ID, true );
        $settings   = isset( $metadata['settings'] ) ? $metadata['settings'] : array();
        $locations  = isset( $metadata['location'] ) ? $metadata['location'] : array(); ?>

        <script type="text/template" id="<?php echo $underscore_template_id; ?>">

        </script>

        <div id="attachments-ui-locations">
            <p><?php _e( 'Define the location of your Attachments Instance; <em>if fields are left empty the limit does not apply.</em>', 'attachmentsui' ); ?></p>
            <?php

                // Post Type(s)
                $options = array();
                $post_types = get_post_types( null, 'objects', null );
                if( is_array( $post_types ) && count( $post_types ) )
                {
                    foreach( $post_types as $cpt_name => $post_type )
                        if( $post_type->public && $cpt_name != 'attachment' )
                            $options[$cpt_name] = $post_type->labels->name;

                    $this->select(
                        $this->meta_key . '[settings][post_type]',
                        __( 'Show if Post Type is', 'attachmentsui' ),
                        $options,
                        isset( $settings['post_type'] ) ? $settings['post_type'] : false,
                        true,
                        __( 'Leave blank to omit this limit', 'attachmentsui' )
                    );
                }

                // Post Format
                if( current_theme_supports( 'post-formats' ) )
                {
                    $options        = array();
                    $post_formats   = get_theme_support( 'post-formats' );

                    if( is_array( $post_formats[0] ) )
                    {
                        foreach( $post_formats[0] as $post_format )
                            $options[$post_format] = ucfirst( $post_format );

                        $this->select(
                            $this->meta_key . '[location][post_formats]',
                            __( 'Show if Post Format is', 'attachmentsui' ),
                            $options,
                            isset( $locations['post_formats'] ) ? $locations['post_formats'] : false,
                            true,
                            __( 'Leave blank to omit this limit', 'attachmentsui' )
                        );
                    }
                }

                // Post IDs
                $this->textfield(
                    $this->meta_key . '[location][posts_in]',
                    __( 'Show if Post ID is', 'attachmentsui' ),
                    isset( $locations['posts_in'] ) && is_array( $locations['posts_in'] ) ? implode( ', ', $locations['posts_in'] ) : '',
                    __( 'Comma separated Post IDs, use negative numbers to represent NOT in', 'attachmentsui' )
                );

                // Post Parent
                $this->textfield(
                    $this->meta_key . '[location][post_parent_in]',
                    __( 'Show if Post Parent is', 'attachmentsui' ),
                    isset( $locations['post_parent_in'] ) && is_array( $locations['post_parent_in'] ) ? implode( ', ', $locations['post_parent_in'] ) : '',
                    __( 'Comma separated Post IDs, use negative numbers to represent NOT in', 'attachmentsui' )
                );

                // User Roles
                if( !empty( $wp_roles->role_names ) )
                {
                    $this->select(
                        $this->meta_key . '[location][roles_in]',
                        __( 'Show if User Role is', 'attachmentsui' ),
                        $wp_roles->role_names,
                        isset( $locations['roles_in'] ) ? $locations['roles_in'] : false,
                        true,
                        __( 'Leave blank to omit this limit', 'attachmentsui' )
                    );
                }

                // Templates
                if( !function_exists( 'get_page_templates' ) )
                    require_once( ABSPATH . 'wp-admin/includes/theme.php' );

                $templates = get_page_templates();
                if( $templates )
                {
                    $options = array();
                    foreach( $templates as $template_name => $template_filename )
                        $options[$template_filename] = htmlentities( $template_name, ENT_QUOTES, $charset );

                    $this->select(
                        $this->meta_key . '[location][template]',
                        __( 'Show if Template is', 'attachmentsui' ),
                        $options,
                        isset( $locations['template'] ) ? $locations['template'] : false,
                        true,
                        __( 'Leave blank to omit this limit', 'attachmentsui' )
                    );
                }

                // Taxonomy Terms
                $taxonomies = get_taxonomies( '', 'objects' );
                $valid = false;
                if( $taxonomies )
                {
                    $options = array();
                    foreach( $taxonomies as $taxonomy )
                    {
                        $terms = get_terms( $taxonomy->name );
                        if( count( $terms ) )
                        {
                            $valid = true;
                            $tax_terms = array();
                            foreach( $terms as $term )
                                $tax_terms[$term->term_id] = $term->name;

                            $options[$taxonomy->name] = array(
                                    'name'      => $taxonomy->name,
                                    'label'     => $taxonomy->label,
                                    'options'   => $tax_terms,
                                );
                        }
                    }

                    if( $valid )
                    {
                        $this->select(
                            $this->meta_key . '[location][terms]',
                            __( 'Show if Term is', 'attachmentsui' ),
                            $options,
                            isset( $locations['terms'] ) ? $locations['terms'] : false,
                            true,
                            __( 'Leave blank to omit this limit', 'attachmentsui' )
                        );
                    }
                }
            ?>
        </div>
    <?php }



    /**
     * Echoes the markup for the Code Reference meta box
     *
     * @param $post WP_Post Provided by WordPress
     */
    function meta_box_markup_code_ref( $post )
    {
        $metadata               = get_post_meta( $post->ID, $this->instance_key . $post->ID, true );
        $instance_fields        = isset( $metadata['fields'] ) ? $metadata['fields'] : array();
		$instance_name 			= empty( $metadata['name'] ) ? $this->instance_key . $post->ID : $metadata['name'];
        ?>
        <p>To pull data from this Instance to your theme, use the following as a starting point:</p>
        <p>Instance: <code><?php echo $instance_name; ?></code></p>
        <p>Here's a code snippet to pull all Attachments from this Instance, you can copy and paste it into your theme template:</p>
<textarea style="display:block; width:100%; height:300px; resize:none; font-family:monospace;">&lt;?php $attachments = new Attachments( '<?php echo $instance_name; ?>' ); ?&gt;
&lt;?php if( $attachments-&gt;exist() ) : ?&gt;
    &lt;h3&gt;Attachments&lt;/h3&gt;
    &lt;p&gt;Total Attachments: &lt;?php echo $attachments-&gt;total(); ?&gt;&lt;/p&gt;
    &lt;ul&gt;
        &lt;?php while( $attachments-&gt;get() ) : ?&gt;
            &lt;li&gt;
                ID: &lt;?php echo $attachments-&gt;id(); ?&gt;&lt;br /&gt;
                Type: &lt;?php echo $attachments-&gt;type(); ?&gt;&lt;br /&gt;
                Subtype: &lt;?php echo $attachments-&gt;subtype(); ?&gt;&lt;br /&gt;
                URL: &lt;?php echo $attachments-&gt;url(); ?&gt;&lt;br /&gt;
                Image: &lt;?php echo $attachments-&gt;image( 'thumbnail' ); ?&gt;&lt;br /&gt;
                Source: &lt;?php echo $attachments-&gt;src( 'full' ); ?&gt;&lt;br /&gt;
                Size: &lt;?php echo $attachments-&gt;filesize(); ?&gt;&lt;br /&gt;<?php foreach( $instance_fields as $instance_field ) : ?>
              <?php echo $instance_field['label']; ?>: &lt;?php echo $attachments-&gt;field( '<?php echo $instance_field['name']; ?>' ); ?&gt;&lt;br /&gt;<?php endforeach; ?>
            &lt;/li&gt;
        &lt;?php endwhile; ?&gt;
    &lt;/ul&gt;
&lt;?php endif; ?&gt;</textarea>
    <?php }



    /**
     * Register the Custom Post Type we'll be using to store Instances
     */
    function register_cpt()
    {
        $labels = array(
            'name'                  => 'Instances',
            'singular_name'         => 'Instance',
            'add_new'               => 'Add New',
            'add_new_item'          => 'Add New Instance',
            'edit_item'             => 'Edit Instance',
            'new_item'              => 'New Instance',
            'all_items'             => 'All Instances',
            'view_item'             => 'View Instance',
            'search_items'          => 'Search Instances',
            'not_found'             => 'No Instances found',
            'not_found_in_trash'    => 'No Instances found in Trash',
            'parent_item_colon'     => '',
        );

        $args = array(
            'labels'                => $labels,
            'public'                => false,
            'publicly_queryable'    => false,
            'show_ui'               => true,
            'show_in_menu'          => false,
            'query_var'             => false,
            'rewrite'               => false,
            'capability_type'       => 'post',
            'has_archive'           => false,
            'hierarchical'          => false,
            'menu_position'         => null,
            'supports'              => array( 'title' )
        );

        register_post_type( $this->cpt_name, $args );
    }



    /**
     * Callback for WordPress admin_menu action, adds our Admin Menu to the Settings Menu
     */
    function wp_admin_menu()
    {
        add_options_page( 'Settings', __( 'Attachments UI', 'attachmentsui' ), 'manage_options', 'edit.php?post_type=attachmentsui' );
    }



    /**
     * Save our Instance metadata
     *
     * @param int $post_id
     * @return mixed
     */
    function save( $post_id )
    {
        // is the user logged in?
        if( !is_user_logged_in() )
            return $post_id;

        // is the nonce set?
        if( !isset( $_POST['attachmentsui_settings_nonce'] ) )
            return $post_id;

        // is the nonce valid?
        if( !wp_verify_nonce( $_POST['attachmentsui_settings_nonce'], 'attachmentsui_save_settings' ) )
            return $post_id;

        // can this user edit this post?
        if( !current_user_can( 'edit_post', $post_id ) )
            return $post_id;

        // passed authentication, proceed with save

        $metadata['settings']   = isset( $_POST[$this->meta_key]['settings'] ) ? $_POST[$this->meta_key]['settings'] : array();
        $metadata['fields']     = isset( $_POST[$this->meta_key]['fields'] ) ? $_POST[$this->meta_key]['fields'] : array();
        $metadata['location']   = isset( $_POST[$this->meta_key]['location'] ) ? $_POST[$this->meta_key]['location'] : array();
        $metadata['append']     = isset( $_POST[$this->meta_key]['append'] ) ? $_POST[$this->meta_key]['append'] : array();

		$metadata['name'] = null;
		if( isset( $_POST[$this->meta_key]['name_override'] ) )
			$metadata['name'] = str_replace( '-', '_', sanitize_title( $_POST[$this->meta_key]['name'] ) );

        // convert CSVs to arrays
        if( !empty( $metadata['location']['posts_in'] ) )
        {
            $posts_in = explode( ',', $metadata['location']['posts_in'] );
            foreach( $posts_in as $key=> $post_in )
                $posts_in[$key] = intval( trim( $post_in ) );

            $metadata['location']['posts_in'] = $posts_in;
        }

        if( !empty( $metadata['location']['post_parent_in'] ) )
        {
            $post_parent_in = explode( ',', $metadata['location']['post_parent_in'] );
            foreach( $post_parent_in as $key=> $post_parent )
                $post_parent_in[$key] = intval( trim( $post_parent ) );

            $metadata['location']['post_parent_in'] = $post_parent_in;
        }

		$metakey = $this->instance_key . $post_id;
		if( !is_null( $metadata['name'] ) )
			$metakey = $metadata['name'];

        update_post_meta( $post_id, $metakey, $metadata );

        return $post_id;
    }



    /**
     * Validate existing Location Limits rules
     *
     * @param bool $val The default value
     * @param string $instance The Instance name
     * @return bool Whether or not all location rules were passed
     */
    function check_rules( $val = true, $instance = null )
    {
        global $post, $current_user;

        // we need to get the location rules for this Instance
        $args = array(
            'posts_per_page'    => 1,
            'post_type'         => $this->cpt_name,
			'post_status'		=> 'publish',
            'meta_query'        => array( array(
                        'key'       => $instance,
                        'compare'   => 'EXISTS',
						'value'		=> null,
                ) )
        );

        $instance_obj = get_posts( $args );

        if( !empty( $instance_obj ) && isset( $instance_obj[0]->ID ) )
        {
            $metadata = get_post_meta( $instance_obj[0]->ID, $instance, true );

            // if we have metadata we need to process the conditions
            if( isset( $metadata['location'] ) )
            {
                $settings = isset( $metadata['settings'] ) ? $metadata['settings'] : false;
                $location = isset( $metadata['location'] ) ? $metadata['location'] : false;

                // Post Type
                if( isset( $settings['post_type'] ) )
                    if( !in_array( $post->post_type, $settings['post_type'] ) )
                        $val = false;

                // Post Format
                if( isset( $location['post_formats'] ) && is_array( $location['post_formats'] ) )
                    if( !in_array( get_post_format( $post->ID ), $location['post_formats'] ) )
                        $val = false;

                // Post ID
                if( isset( $location['posts_in'] ) && is_array( $location['posts_in'] ) )
                {
                    // negative post IDs represent "NOT"
                    $posts_in       = array();
                    $posts_not_in   = array();

                    foreach( $location['posts_in'] as $post_id )
                        if( $post_id > 0 )
                            $posts_in[] = $post_id;
                        else
                            $posts_not_in[] = abs( $post_id );

                    if( ( !empty( $posts_in ) && !in_array( $post->ID, $posts_in ) ) || ( !empty( $posts_not_in ) && in_array( $post->ID, $posts_not_in ) ) )
                        $val = false;
                }

                // Post Parent
                if( isset( $location['post_parent_in'] ) && is_array( $location['post_parent_in'] ) )
                {
                    // negative post IDs represent "NOT"
                    $posts_in       = array();
                    $posts_not_in   = array();

                    foreach( $location['post_parent_in'] as $post_id )
                        if( $post_id > 0 )
                            $posts_in[] = $post_id;
                        else
                            $posts_not_in[] = abs( $post_id );

                    if( ( !empty( $posts_in ) && !in_array( $post->post_parent, $posts_in ) ) || ( !empty( $posts_not_in ) && in_array( $post->post_parent, $posts_not_in ) ) )
                        $val = false;
                }

                // User Role
                if( isset( $location['roles_in'] ) && is_array( $location['roles_in'] ) )
                {
                    get_currentuserinfo();
                    $user  = new WP_User( $current_user->ID );
                    if( is_array( $user->roles ) )
                    {
                        $roles = array_intersect( $user->roles, $location['roles_in'] );
                        if( empty( $roles ) )
                            $val = false;
                    }
                }

                // Template
                if( isset( $location['template'] ) && is_array( $location['template'] ) )
                {
                    foreach( $location['template'] as $key => $template )
                        $location['template'][$key] = trailingslashit( get_template_directory() ) . $template;

                    if( !in_array( get_page_template(), $location['template'] ) )
                        $val = false;
                }

                // Term
                if( isset( $location['terms'] ) && is_array( $location['terms'] ) )
                {
                    $post_terms = array();

                    $taxonomies = get_taxonomies( '', 'objects' );
                    if( $taxonomies )
                    {
                        foreach( $taxonomies as $taxonomy )
                        {
                            $terms = wp_get_post_terms( $post->ID, $taxonomy->name, array( 'fields' => 'ids' ) );
                            if( is_array( $terms ) && !empty( $terms ) )
                                $post_terms = array_unique( array_merge( $post_terms, $terms ) );
                        }

                        $found = array_intersect( $post_terms, $location['terms'] );
                        if( !$found )
                            $val = false;

                    }
                }
            }
        }

        return $val;
    }
}



/**
 * Instantiate Attachments UI
 *
 * @param Attachments $attachments The Attachments singleton
 */
function init_attachments_ui( $attachments )
{
    // instantiate singleton
    new AttachmentsUI( $attachments );
}

add_action( 'attachments_extension', 'init_attachments_ui', 10, 1 );



/**
 * Outputs admin notice if Attachments is not active
 */
function aui_admin_notice()
{
	if( !is_plugin_active( 'attachments/index.php' ) || !class_exists( 'Attachments' ) ): ?>
	<div class="updated">
		<p><?php _e( 'Attachments must be installed and activated for Attachments UI to work.', 'attachmentsui' ); ?></p>
	</div>
<?php endif; }

add_action( 'admin_notices', 'aui_admin_notice' );
